
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : base.scm
;; DESCRIPTION : frequently used TeXmacs-independent Scheme subroutines
;; COPYRIGHT   : (C) 2002  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Output
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (display-list l)
  (if (not (null? l))
      (begin
	(display (car l))
	(display-list (cdr l)))))

(define display*
  (lambda l (display-list l)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Utility
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (c1r l) (car l))
(define (c2r l) (cadr l))
(define (c3r l) (caddr l))
(define (c4r l) (cadddr l))
(define (c11r l) (caar l))
(define (c21r l) (cadar l))
(define (c31r l) (caddar l))
(define (c12r l) (caadr l))
(define (c22r l) (cadadr l))
(define (c13r l) (caaddr l))
(define (c111r l) (caaar l))
(define (c211r l) (cadaar l))
(define (c121r l) (caadar l))
(define (c112r l) (caaadr l))
(define (c1111r l) (caaaar l))

(define (cAr l) (car (reverse l)))
(define (cDr l) (reverse (cdr (reverse l))))
(define (cADr l) (cadr (reverse l)))
(define (cDDr l) (reverse (cddr (reverse l))))
(define (cDdr l) (cDr (cdr l)))

(define cons*
  (lambda l
    (let ((r (reverse l)))
      (append (reverse (cdr r)) (car r)))))

(define (rcons l x)
  (append l (list x)))

(define rcons*
  (lambda l
    (append (car l) (cdr l))))

(define (map-unary f l)
  (if (null? l) l
      (cons (f (car l)) (map-unary f (cdr l)))))

(define (exec-unary f l)
  (if (not (null? l))
      (begin
	(f (car l))
	(exec-unary f (cdr l)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Lists
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (in? x l)
  (cond ((null? l) #f)
	((equal? (car l) x) #t)
	(else (in? x (cdr l)))))

(define (list-starts? l what)
  (cond ((null? what) #t)
	((null? l) #f)
	(else (and (equal? (car l) (car what))
		   (list-starts? (cdr l) (cdr what))))))

(define (sublist l i j)
  (if (> i 0)
      (if (null? l) l (sublist (cdr l) (- i 1) (- j 1)))
      (if (> j 0)
	  (if (null? l) l (cons (car l) (sublist (cdr l) 0 (- j 1))))
	  '())))

(define (list-replace l what by)
  (cond ((null? l) l)
	((list-starts? l what)
	 (let ((tail (sublist l (length what) (length l))))
	   (append by (list-replace tail what by))))
	(else (cons (car l) (list-replace (cdr l) what by)))))

(define (split l pred)
  (if (null? l) (list l l)
      (if (pred (car l))
	  (list '() l)
	  (let ((p (split (cdr l) pred)))
	    (list (cons (car l) (car p)) (cadr p))))))

(define (filter l pred?)
  (cond ((null? l) l)
	((pred? (car l)) (cons (car l) (filter (cdr l) pred?)))
	(else (filter (cdr l) pred?))))

(define (list-sort-insert x l comp)
  (cond ((null? l) (list x))
	((comp x (car l)) (cons x l))
	(else (cons (car l) (list-sort-insert x (cdr l) comp)))))

(define (list-sort l comp)
  (if (null? l) l
      (let ((r (list-sort (cdr l) comp)))
	(list-sort-insert (car l) r comp))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Booleans
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (xor-sub l)
  (cond ((null? l) #f)
	((car l) (not (xor-sub (cdr l))))
	(else (xor-sub (cdr l)))))

(define xor (lambda l (xor-sub l)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Strings
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (char->string c) (list->string (list c)))

(define (char-in-string? c s)
  (not (equal? (string-index s c) #f)))

(define (string-starts? s what)
  (let ((n (string-length s))
	(k (string-length what)))
    (and (>= n k) (equal? (substring s 0 k) what))))

(define (string-ends? s what)
  (let ((n (string-length s))
	(k (string-length what)))
    (and (>= n k) (equal? (substring s (- n k) n) what))))

(define (string-replace s what by)
  (list->string
   (list-replace
    (string->list s)
    (string->list what)
    (string->list by))))

(define (force-string s)
  (if (string? s) s ""))

(define (func? . l)
  (let ((n (length l)))
    (cond ((= n 2)
	   (let ((x (car l)) (f (cadr l)))
	     (and (list? x) (not (null? x)) (equal? (car x) f))))
	  ((= n 3)
	   (let ((x (car l)) (f (cadr l)) (nn (caddr l)))
	     (and (list? x) (not (null? x))
		  (equal? (car x) f) (= (length x) (+ nn 1)))))
	  (else #f))))

(define (tuple? . args)
  (if (null? (cdr args)) (list? (car args)) (apply func? args)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Patterns
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (match-list? l pattern)
  (cond ((null? pattern) (null? l))
	((number? (car pattern))
	 (and (>= (length l) (car pattern))
	      (match-list? (sublist l (car pattern) (length l))
			   (cdr pattern))))
	((equal? pattern '(*)) #t)
	(else (and (match? (car l) (car pattern))
		   (match-list? (cdr l) (cdr pattern))))))

(define (match-sub? x pattern)
  (let ((first (car pattern)))
    (cond ((equal? first 'and)
	   (and (match? x (c2r pattern)) (match? x (c3r pattern))))
	  ((equal? first 'or)
	   (or (match? x (c2r pattern)) (match? x (c3r pattern))))
	  ((equal? first 'not)
	   (not (match? x (c2r pattern))))
	  ((equal? first 'quote) (equal? x (c2r pattern)))
	  ((list? x) (match-list? x pattern))
	  (else #f))))

(define (match? x pattern)
  (cond ((null? pattern) (null? x))
	((list? pattern) (match-sub? x pattern))
	((and (symbol? pattern) (string-ends? (symbol->string pattern) "?"))
	 (eval (list pattern x)))
	(else (equal? x pattern))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Dictionaries and sets
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (fill-dictionary-entry d key im)
  (if (not (null? key))
      (begin
	(hash-set! d (car key) im)
	(fill-dictionary-entry d (cdr key) im))))

(define (fill-dictionary d l)
  (if (not (null? l))
      (begin
	(let* ((r (reverse (car l))))
	  (fill-dictionary-entry d (cdr r) (car r)))
	(fill-dictionary d (cdr l)))))

(define (fill-set d l)
  (if (not (null? l))
      (begin
	(hash-set! d (car l) #t)
	(fill-set d (cdr l)))))
