
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : basic-menus.scm
;; DESCRIPTION : basic menus
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The File menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define print-menu
  '(("Preview with ghostview" (preview-with-ghostview))
    ---
    ("Print all" (print))
    ("Print page selection" ...
     (interactive '("First page:" "Last page:") 'print-pages))
    ("Print all to file" ...
     (choose-file "Print all to file" "postscript" 'print-to-file))
    ("Print page selection to file" ...
     (interactive '("First page:" "Last page:")
		  'choose-file-and-print-page-selection))))

(define file-menu
  '(("New" (new-buffer))
    ("Load" ... (choose-file "Load file" "" 'load-buffer))
    ("Save" (save-buffer))
    ("Save as" ...
     (choose-file "Save TeXmacs file" "TeXmacs" 'save-buffer))
    ("Revert" ... (revert-buffer))
    ---
    (-> "Print" (link print-menu))
    (-> "Import"
	("Verbatim" ...
	 (choose-file "Load verbatim file" "" 'load-verbatim-buffer))
	("LaTeX" ... (choose-file "Load LaTeX file" "latex"
				  'load-latex-buffer))
	("HTML" ... (choose-file "Load html file" "html" 'load-html-buffer))
	("Scheme" ...
	 (choose-file "Load scheme file" "scheme" 'load-scheme-buffer)))
    (-> "Export"
	("Verbatim" ...
	 (choose-file "Save verbatim file" "" 'save-verbatim-buffer))
	("LaTeX" ...
	 (choose-file "Save LaTeX file" "latex" 'save-latex-buffer))
	("HTML" ...
	 (choose-file "Save HTML file" "html" 'save-html-buffer))
	("Scheme" ...
	 (choose-file "Save scheme file" "scheme" 'save-scheme-buffer))
	("PostScript" ...
	 (choose-file "File postscript file" "postscript" 'export-postscript)))
    ---
    ("Close buffer" (kill-buffer))
    ("Close TeXmacs" (safely-quit-TeXmacs))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Go menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define bookmarks-menu
  '())

(define go-menu
  '((link buffer-menu)
    (if (not (null? bookmarks-menu))
	---
	(link bookmarks-menu))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Edit menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define edit-menu
  '(("Undo" (undo))
    ("Redo" (redo))
    ---
    (when (selection-active-any?)
	  ("Copy" (copy "primary"))
	  ("Cut" (cut "primary")))
    ("Paste" (paste "primary"))
    ("Clear" (selection-clear "primary"))
    ---
    ("Search" ... (search-start #t))
    ("Replace" ...
     (interactive '("Replace:" "Replace by:") 'replace-start-forward))
    ("Spell" ... (spell-start))
    ---
    (when (selection-active-any?)
	  (-> "Copy to"
	      ("Primary" (copy "primary"))
	      ("Secondary" (copy "secondary"))
	      ("Ternary" (copy "ternary"))
	      ---
	      ("Other" ... (interactive '("Copy to:") 'copy)))
	  (-> "Cut to"
	      ("Primary" (cut "primary"))
	      ("Secondary" (cut "secondary"))
	      ("Ternary" (cut "ternary"))
	      ---
	      ("Other" ... (interactive '("Cut to:") 'cut))))
    (-> "Paste from"
	("Primary" (paste "primary"))
	("Secondary" (paste "secondary"))
	("Ternary" (paste "ternary"))
	---
	("Other" ... (interactive '("Paste from:") 'paste)))
    ---
    (-> "Preferences"
	(link preferences-menu))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Project menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define project-menu
  '((link project-buffer-menu)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The View menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define view-menu
  '(("Open new window" (open-window))
    ("Clone window" (clone-window))
    ("Load in new window" ... (choose-file "Load file" "" 'load-in-new-window))
    ("Close window" (kill-window))
    ---
    ("Header" (toggle-visible-header))
    (when (visible-header?)
	  ("Main icon bar" (toggle-visible-icon-bar 0))
	  ("Context dependent icons" (toggle-visible-icon-bar 1))
	  ("User provided icons" (toggle-visible-icon-bar 2)))
    ("Status bar" (toggle-visible-footer))
    ---
    ("Presentation mode" (toggle-full-screen-mode))
    (-> "Shrinking factor"
	("Default" (init-default "shrinking factor"))
	---
	("1" (set-shrinking-factor 1))
	("2" (set-shrinking-factor 2))
	("3" (set-shrinking-factor 3))
	("4" (set-shrinking-factor 4))
	("5" (set-shrinking-factor 5))
	("7" (set-shrinking-factor 7))
	("10" (set-shrinking-factor 10))
	---
	("Other" ...
	 (interactive '("Shrinking factor:") 'other-shrinking-factor)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Preferences menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define preferences-tree
  '((enum ("Look and feel" "look and feel")
	  ("Emacs" "emacs")
;	  ("Windows" "windows")
	  ("Old style" "old style"))
;   (enum ("Profile" "profile")
;	  ("Beginner" "beginner")
;	  ("Normal" "normal")
;	  ("Advanced" "advanced"))
    (enum ("Language" "language")
	  ("British" "british")
	  ("Czech" "czech")
	  ("Dutch" "dutch")
	  ("English" "english")
	  ("French" "french")
	  ("German" "german")
	  ("Hungarian" "hungarian")
	  ("Italian" "italian")
	  ("Polish" "polish")
	  ("Portuguese" "portuguese")
	  ("Romanian" "romanian")
	  ("Russian" "russian")
	  ("Spanish" "spanish")
	  ("Swedish" "swedish")
	  ("Ukrainian" "ukrainian"))
    (-> "Keyboard"
	(enum ("A modifier" "A")
	      ("Default" "default")
	      ("Equivalent for#Mod1" "Mod1")
	      ("Equivalent for#Mod2" "Mod2")
	      ("Equivalent for#Mod3" "Mod3")
	      ("Equivalent for#Mod4" "Mod4")
	      ("Equivalent for#Mod5" "Mod5"))
	(enum ("M modifier" "M")
	      ("Default" "default")
	      ("Equivalent for#Mod1" "Mod1")
	      ("Equivalent for#Mod2" "Mod2")
	      ("Equivalent for#Mod3" "Mod3")
	      ("Equivalent for#Mod4" "Mod4")
	      ("Equivalent for#Mod5" "Mod5"))
	(enum ("H modifier" "H")
	      ("Default" "default")
	      ("Equivalent for#Mod1" "Mod1")
	      ("Equivalent for#Mod2" "Mod2")
	      ("Equivalent for#Mod3" "Mod3")
	      ("Equivalent for#Mod4" "Mod4")
	      ("Equivalent for#Mod5" "Mod5"))
	---
	(enum ("Alt key" "alt")
	      ("Don't remap" "default")
	      ("Map to#A modifier" "A")
	      ("Map to#M modifier" "M")
	      ("Map to#H modifier" "H"))
	(enum ("Meta key" "meta")
	      ("Don't remap" "default")
	      ("Map to#A modifier" "A")
	      ("Map to#M modifier" "M")
	      ("Map to#H modifier" "H"))
	(enum ("Windows key" "windows")
	      ("Don't remap" "default")
	      ("Map to#A modifier" "A")
	      ("Map to#M modifier" "M")
	      ("Map to#H modifier" "H"))
	(enum ("Caps-lock key" "caps-lock")
	      ("Don't remap" "default")
	      ("Map to#A modifier" "A")
	      ("Map to#M modifier" "M")
	      ("Map to#H modifier" "H"))
	---
	(enum ("Cyrillic input method" "cyrillic input method")
	      ("Translit" "translit")
	      ("Jcuken" "jcuken")
	      ("Yawerty" "yawerty")
	      ("Koi8-r" "koi8-r")
	      ("Cp1251" "cp1251")))
    (-> "Printer"
	(string ("Printing command" "printing command"))
	(string ("Paper type" "paper type"))
	(string ("Printer dpi" "printer dpi")))
    (-> "View"
	(enum ("Header" "header")
	      ("On" "on")
	      ("Off" "off"))
	(enum ("Main icon bar" "main icon bar")
	      ("On" "on")
	      ("Off" "off"))
	(enum ("Context dependent icons" "context dependent icons")
	      ("On" "on")
	      ("Off" "off"))
	(enum ("User provided icons" "user provided icons")
	      ("On" "on")
	      ("Off" "off"))
	(enum ("Status bar" "status bar")
	      ("On" "on")
	      ("Off" "off"))
	(enum ("Shrinking factor" "shrinking factor")
	      "1" "2" "3" "4" "5" "7" "10" *))
    (enum ("Security" "security")
	  ("Accept no scripts" "accept no scripts")
	  ("Prompt on scripts" "prompt on scripts")
	  ("Accept all scripts" "accept all scripts"))))

(define preferences-menu '())

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Tools menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define tools-menu
  '((-> "Execute"
	("Execute system command" ... (interactive '("Command:") 'system))
	("Evaluate scheme expression" ...
	 (interactive '("Command:") 'TeXmacs-eval)))
    (-> "Selections"
	(-> "Import"
	    ("TeXmacs" (selection-set-import "TeXmacs"))
	    ("LaTeX" (selection-set-import "latex"))
	    ("HTML" (selection-set-import "html"))
	    ("Scheme" (selection-set-import "scheme"))
	    ("Verbatim" (selection-set-import "verbatim")))
	(-> "Export"
	    ("TeXmacs" (selection-set-export "TeXmacs"))
	    ("LaTeX" (selection-set-export "latex"))
	    ("HTML" (selection-set-export "html"))
	    ("Scheme" (selection-set-export "scheme"))
	    ("Verbatim" (selection-set-export "verbatim"))))
    (-> "Update"
	("Image links" (postscript-gc))
	("Inclusions" (inclusions-gc))
	("Styles" (style-clear-cache)))
    (if (not (null? test-menu))
	---
	(-> "Test" (link test-menu)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Help menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define help-menu
  '(("Welcome" (load-help-buffer "help/welcome"))
    ---
    ("Configuration" (load-help-buffer "help/configure"))
    ("Introduction" (load-help-buffer "help/introduction"))
    ("Keyboard" (load-help-buffer "help/keyboard"))
    ("Tables" (load-help-buffer "help/tables"))
    ("Advanced" (load-help-buffer "help/advanced"))
    ("Latex" (load-help-buffer "help/compatibility"))
    ---
    ("Styles" (load-help-buffer "help/styles"))
    ("Scheme" (load-help-buffer "help/scheme"))
    ("Sessions" (load-help-buffer "help/session"))
    ---
    (-> "About"
	("About" (load-help-buffer "help/about"))
	("GNU#license" (load-verbatim-buffer "$TEXMACS_PATH/LICENSE"))
	---
	("First start" (load-help-buffer "startup/readme"))
	(if (file-exists? "$TEXMACS_HOME_PATH"
			  "doc/english/startup/upgrade-last.tex")
	    ("What is new" (load-help-buffer "startup/upgrade-last")))
	("Major changes" (load-help-buffer "startup/upgrade-hist")))
    (-> "Language"
	("Russian" (load-help-buffer "help/russian")))
    (-> "Interfacing"
	("The mycas example" (load-help-buffer "help/itf_example"))
	("Your first interface" (load-help-buffer "help/itf_first"))
	("Inclusion into TeXmacs" (load-help-buffer "help/itf_include"))
	("Further customization" (load-help-buffer "help/itf_customize"))
	("Dynamic libraries" (load-help-buffer "help/itf_protocol"))
	("Future plans" (load-help-buffer "help/itf_projects")))
    (-> "Implementation"
	("Architecture" (load-help-buffer "source/general"))
	---
	("Basic types" (load-help-buffer "source/basic"))
	("Data format" (load-help-buffer "source/data"))
	("Data conversion" (load-help-buffer "source/convert"))
	("Graphical interface" (load-help-buffer "source/window"))
	("Fonts" (load-help-buffer "source/fonts"))
	("Mathematics" (load-help-buffer "source/math"))
	("Typesetted boxes" (load-help-buffer "source/boxes"))
	---
	("Known bugs" (load-help-buffer "source/bugs")))
    (-> "Projects"
	("Past changes" (load-help-buffer "projects/changes"))
	("Future changes" (load-help-buffer "projects/tochange"))
	("Other projects" (load-help-buffer "projects/projects")))
    (-> "Help me"
	("Please" (load-help-buffer "projects/helpme")))
    ---
    ("Online documentation"
     (load-buffer "http://www.gnu.org/software/texmacs-doc/index.en.tm"))))
