
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : format-menus.scm
;; DESCRIPTION : menus for setting local and global formatting properties
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Font submenu in text mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define text-font-menu
  '((-> "Name"
	("Roman" (make-with "font" "roman"))
	(if (tex-file-exists? "pnr10.mf")
	    ("Pandora" (make-with "font" "pandora")))
	(if (tex-file-exists? "ccr10.mf")
	    ("Concrete" (make-with "font" "concrete")))
	---
	(-> "Adobe"
	    (if (tex-file-exists? "rpagk.tfm")
		("Avant Garde" (make-with "font" "avant-garde")))
	    (if (tex-file-exists? "rpbkl.tfm")
		("Bookman" (make-with "font" "bookman")))
	    (if (tex-file-exists? "rpzcmi.tfm")
		("Chancery" (make-with "font" "chancery")))
	    (if (tex-file-exists? "rpcrr.tfm")
		("Courier" (make-with "font" "courier")))
	    (if (tex-file-exists? "rpzdr.tfm")
		("Dingbat" (make-with "font" "dingbat")))
	    (if (tex-file-exists? "rphvr.tfm")
		("Helvetica" (make-with "font" "helvetica")))
	    (if (tex-file-exists? "rpncr.tfm")
		("New Century Schoolbook"
		  (make-with "font" "new-century-schoolbook")))
	    (if (tex-file-exists? "rpplr.tfm")
		("Palatino" (make-with "font" "palatino")))
	    (if (tex-file-exists? "rptmr.tfm")
		("Times" (make-with "font" "times"))))
	(-> "Latin"
	    (if (tex-file-exists? "callig15.mf")
		("Calligraphic" (make-with "font" "calligraphic")))
	    (if (tex-file-exists? "capbas.mf")
		("Capbas" (make-with "font" "capbas")))
	    (if (tex-file-exists? "cdr10.mf")
		("Duerer" (make-with "font" "duerer")))
	    (if (tex-file-exists? "hscs10.mf")
		("Hershey" (make-with "font" "hershey")))
	    (if (tex-file-exists? "la14.mf")
		("La" (make-with "font" "la")))
	    (if (tex-file-exists? "cmfi10.mf")
		("Messy" (make-with "font" "messy")))
	    (if (tex-file-exists? "ocr10.mf")
		("Optical" (make-with "font" "optical")))
	    (if (tex-file-exists? "cpcr10.mf")
		("Pacioli" (make-with "font" "pacioli")))
	    (if (tex-file-exists? "punk20.mf")
		("Punk" (make-with "font" "punk")))
	    (if (tex-file-exists? "twcal14.mf")
		("Tw#Calligraphic" (make-with "font" "twcal")))
	    (if (tex-file-exists? "va14.mf")
		("Va" (make-with "font" "va"))))
	(-> "Gothic"
	    (if (tex-file-exists? "blackletter.mf")
		("Blackletter" (make-with "font" "blackletter")))
	    (if (tex-file-exists? "eufm10.mf")
		("Euler" (make-with "font" "Euler")))
	    (if (tex-file-exists? "ygoth.mf")
		("Gothic" (make-with "font" "gothic")))
	    (if (tex-file-exists? "hge.mf")
		("Old English" (make-with "font" "old-english")))
	    (if (tex-file-exists? "schwell.mf")
		("Schwell" (make-with "font" "schwell")))
	    (if (tex-file-exists? "suet14.mf")
		("Suet" (make-with "font" "suet")))
	    (if (tex-file-exists? "yswab.mf")
		("Swab" (make-with "font" "swab"))))
	(-> "Foreign"
	    (if (tex-file-exists? "nash14.mf")
		("Arab" (make-with "font" "arab")))
	    (if (tex-file-exists? "artmr10.mf")
		("Armenian" (make-with "font" "armenian")))
	    ("Cyrillic" (make-with "font" "cyrillic"))
	    (if (tex-file-exists? "dvng10.mf")
		("Devangari" (make-with "font" "devangari")))
	    (if (tex-file-exists? "mxed.mf")
		(-> "Georgian"
		    ("Mxedruli" (make-with "font" "mxedruli"))
		    ("Xucuri" (make-with "font" "xucuri"))))
	    (if (tex-file-exists? "mrgrrg10.mf")
		("Greek" (make-with "font" "greek")))
	    (if (tex-file-exists? "redis.mf")
		("Hebrew" (make-with "font" "hebrew")))
	    (if (tex-file-exists? "imr10.mf")
		("Icelandic" (make-with "font" "icelandic")))
	    (if (tex-file-exists? "eiadr10.mf")
		("Irish" (make-with "font" "irish")))
	    (if (tex-file-exists? "osmanian.mf")
		("Osmanian" (make-with "font" "osmanian")))
	    (if (tex-file-exists? "wtkr10.mf")
		("Turkish" (make-with "font" "turkish")))
	    (if (tex-file-exists? "wntml10.mf")
		("Tamil" (make-with "font" "tamil")))
	    (if (tex-file-exists? "thairz10.mf")
		("Thai" (make-with "font" "thai")))
	    (if (tex-file-exists? "vmr10.mf")
		("Vietnamese" (make-with "font" "vietnamese"))))
	(-> "Archaic"
	    (if (tex-file-exists? "bard.mf")
		("Bard" (make-with "font" "bard")))
	    (if (tex-file-exists? "cypr10.mf")
		("Cypriot" (make-with "font" "cypriot")))
	    (if (tex-file-exists? "etr10.mf")
		("Etruscan" (make-with "font" "etruscan")))
	    (if (tex-file-exists? "givbc10.mf")
		(-> "Greek"
		    ("4tH Century BC" (make-with "font" "greek4cbc"))
		    ("6th Century BC" (make-with "font" "greek6cbc"))))
	    (if (tex-file-exists? "linb10.mf")
		("Linear Beta" (make-with "font" "linearb")))
	    (if (tex-file-exists? "ogham.mf")
		("Ogham" (make-with "font" "ogham")))
	    (if (tex-file-exists? "phnc10.mf")
		("Phoenician" (make-with "font" "phoenician")))
	    (if (tex-file-exists? "fut10.mf")
		(-> "Runic"
		    ("Default" (make-with "font" "runic"))
		    ("Futhark" (make-with "font" "runic*"))
		    ("Futhork" (make-with "font" "runic**"))))
	    ;; ("south arabian" (make-with "font" "southarabian"))
	    ;; ("syriac" (make-with "font" "syriac"))
	    (if (tex-file-exists? "izhitsa.mf")
		("Old Slavonic" (make-with "font" "old-slavonic")))
	    (if (tex-file-exists? "ugaritic.mf")
		("Ugaritic" (make-with "font" "ugaritic"))))
	(-> "Phantasy"
	    (if (tex-file-exists? "cherokee.mf")
		("Cherokee" (make-with "font" "cherokee")))
	    (if (tex-file-exists? "shavian.mf")
		("Shavian" (make-with "font" "shavian")))
	    (if (tex-file-exists? "tengwar.mf")
		("Tengwar" (make-with "font" "tengwar"))))
	(-> "Miscellaneous"
	    (if (tex-file-exists? "dancers.mf")
		("Dancers" (make-with "font" "dancers")))
	    (if (tex-file-exists? "bbding10.mf")
		("Dingbat" (make-with "font" "bbding")))
	    (if (tex-file-exists? "go10.mf")
		("Go" (make-with "font" "go")))
	    (if (tex-file-exists? "iching.mf")
		("Iching" (make-with "font" "iching")))
	    (if (tex-file-exists? "karta15.mf")
		("Karta" (make-with "font" "karta")))
	    (if (tex-file-exists? "klinz.mf")
		("Klinz" (make-with "font" "klinz")))
	    (if (tex-file-exists? "magic.mf")
		("Magic" (make-with "font" "magic")))
	    (if (tex-file-exists? "cmph10.mf")
		("Phonetic" (make-with "font" "phonetic")))
	    (if (tex-file-exists? "tsipa10.mf")
		("Tsipa" (make-with "font" "tsipa")))
	    (if (tex-file-exists? "wsuipa10.mf")
		("Wsuipa" (make-with "font" "wsuipa"))))
	(-> "X-windows"
	    ("Times" (make-with "font" "x-times"))
	    ("Courier" (make-with "font" "x-courier"))
	    ("Helvetica" (make-with "font" "x-helvetica"))
	    ("Utopia" (make-with "font" "x-utopia"))
	    ("Lucida" (make-with "font" "x-lucida"))))
    (-> "Variant"
	("Roman" (make-with "font family" "rm"))
	("Typewriter" (make-with "font family" "tt"))
	("Sans serif" (make-with "font family" "ss")))
    (-> "Series"
	("Light" (make-with "font series" "light"))
	("Medium" (make-with "font series" "medium"))
	("Bold" (make-with "font series" "bold")))
    (-> "Shape"
	("Right" (make-with "font shape" "right"))
	("Slanted" (make-with "font shape" "slanted"))
	("Italic" (make-with "font shape" "italic"))
	("Left slanted" (make-with "font shape" "left-slanted"))
	---
	("Small caps" (make-with "font shape" "small-caps"))
	("Proportional" (make-with "font shape" "proportional"))
	("Condensed" (make-with "font shape" "condensed"))
	("Flat" (make-with "font shape" "flat"))
	("Long" (make-with "font shape" "long")))
    (-> "Size" (link font-size-menu))))

(define font-size-menu
  '(("Tiny" (make-with "font size" "0.59"))
    ("Very small" (make-with "font size" "0.71"))
    ("Small" (make-with "font size" "0.84"))
    ("Normal" (make-with "font size" "1"))
    ("Large" (make-with "font size" "1.19"))
    ("Very large" (make-with "font size" "1.41"))
    ("Huge" (make-with "font size" "1.68"))
    ("Really huge" (make-with "font size" "2"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Font submenu in math mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define math-font-menu
  '((-> "Name"
	("Roman" (make-with "math font" "roman"))
	(if (tex-file-exists? "ccr10.mf")
	    ("Concrete" (make-with "math font" "concrete")))
	(if (tex-file-exists? "eurm10.mf")
	    ("New Roman" "E-C-E" (make-with "math font" "ENR")))
	---
	(if (tex-file-exists? "rpsyr.tfm")
	    ("Adobe" (make-with "math font" "adobe")))
	(if (tex-file-exists? "cdr10.mf")
	    ("Duerer" (make-with "math font" "Duerer")))
	(if (tex-file-exists? "eufm10.mf")
	    ("Euler" (make-with "math font" "Euler")))
	(-> "Calligraphic"
	    ("Default" (make-with "math font" "cal"))
	    (if (tex-file-exists? "euxm10.mf")
		("Euler" (make-with "math font" "cal**")))
	    (if (tex-file-exists? "rsfs10.mf")
		("Ralph Smith's" (make-with "math font" "cal*"))))
	(-> "Blackboard bold"
	    ("Default" (make-with "math font" "Bbb*"))
	    (if (tex-file-exists? "msbm10.mf")
		("A.M.S." (make-with "math font" "Bbb")))
	    (if (tex-file-exists? "bbold10.mf")
		("Blackboard bold" (make-with "math font" "Bbb**")))
	    (if (tex-file-exists? "ocmr10.mf")
		("Outlined roman" (make-with "math font" "Bbb***")))
	    (if (tex-file-exists? "dsrom10.mf")
		("Double stroke" (make-with "math font" "Bbb****")))))
    (if (real-math-font? (get-env "math font"))
	(-> "Variant"
	    ("Roman" (make-with "math font family" "mr"))
	    ("Typewriter" (make-with "math font family" "mt"))
	    ("Sans serif" (make-with "math font family" "ms"))
	    ---
	    (-> "Text font"
		("Roman" (make-with "math font family" "rm"))
		("Typewriter" (make-with "math font family" "tt"))
		("Sans serif" (make-with "math font family" "ss"))
		("Bold" (make-with "math font family" "bf"))
		("Right" (make-with "math font family" "up"))
		("Slanted" (make-with "math font family" "sl"))
		("Italic" (make-with "math font family" "it"))))
	(if (real-math-family? (get-env "math font family"))
	    (-> "Series"
		("Light" (make-with "math font series" "light"))
		("Medium" (make-with "math font series" "medium"))
		("Bold" (make-with "math font series" "bold")))))
    (if (not (real-math-font? (get-env "math font")))
	(-> "Variant"
	    ("Roman" (make-with "math font family" "mr"))
	    ("Typewriter" (make-with "math font family" "mt"))
	    ("Sans serif" (make-with "math font family" "ms")))
	(-> "Series"
	    ("Medium" (make-with "math font series" "medium"))
	    ("Bold" (make-with "math font series" "bold")))
	(-> "Shape"
	    ("Default" (make-with "math font shape" "normal"))
	    ("Right" (make-with "math font shape" "right"))
	    ("Slanted" (make-with "math font shape" "slanted"))
	    ("Italic" (make-with "math font shape" "italic"))))
    (-> "Size" (link font-size-menu))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Font submenu in prog mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define prog-font-menu
  '((-> "Name"
	("roman" (make-with "prog font" "roman"))
	(if (tex-file-exists? "ccr10.mf")
	    ("concrete" (make-with "prog font" "concrete")))
	(if (tex-file-exists? "pnr10.mf")
	    ("pandora" (make-with "prog font" "pandora"))))
    (-> "Variant"
	("Roman" (make-with "prog font family" "rm"))
	("Typewriter" (make-with "prog font family" "tt"))
	("Sans serif" (make-with "prog font family" "ss")))
    (-> "Series"
	("Medium" (make-with "prog font series" "medium"))
	("Bold" (make-with "prog font series" "bold")))
    (-> "Shape"
	("Default" (make-with "prog font shape" "normal"))
	("Right" (make-with "prog font shape" "right"))
	("Slanted" (make-with "prog font shape" "slanted"))
	("Italic" (make-with "prog font shape" "italic")))
    (-> "Size" (link font-size-menu))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Color and Language submenus
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define color-menu
  '(("Black" (make-with "color" "black"))
    ("White" (make-with "color" "white"))
    ("Grey" (make-with "color" "grey"))
    ("Red" (make-with "color" "red"))
    ("Blue" (make-with "color" "blue"))
    ("Yellow" (make-with "color" "yellow"))
    ("Green" (make-with "color" "green"))
    ("Orange" (make-with "color" "orange"))
    ("Magenta" (make-with "color" "magenta"))
    ("Brown" (make-with "color" "brown"))
    ("Pink" (make-with "color" "pink"))))

(define language-menu
  '(("British" (make-with "language" "british"))
    ("Czech" (make-with "language" "czech"))
    ("Dutch" (make-with "language" "dutch"))
    ("English" (make-with "language" "english"))
    ("French" (make-with "language" "french"))
    ("German" (make-with "language" "german"))
    ("Hungarian" (make-with "language" "hungarian"))
    ("Italian" (make-with "language" "italian"))
    ("Polish" (make-with "language" "polish"))
    ("Portuguese" (make-with "language" "portuguese"))
    ("Romanian" (make-with "language" "romanian"))
    ("Russian"
     (begin
       (make-with "language" "russian")
       (make-with "font" "cyrillic")))
    ("Spanish" (make-with "language" "spanish"))
    ("Swedish" (make-with "language" "swedish"))
    ("Ukrainian"
     (begin
       (make-with "language" "ukrainian")
       (make-with "font" "cyrillic")))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Extra formatting submenus for math mode
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define math-format-menu
  '((-> "Index level"
	("Normal" (make-with "index level" "0"))
	("Script size" (make-with "index level" "1"))
	("Script script size" (make-with "index level" "2")))
    (-> "Formula style"
	("on" (make-with "formula style" "true"))
	("off" (make-with "formula style" "false")))
    (-> "Condensed"
	("on" (make-with "math condensed" "true"))
	("off" (make-with "math condensed" "false")))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Formatting of paragraphs
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define paragraph-menu
  '((-> "Alignment"
	("Justified" (make-line-with "paragraph mode" "justify"))
	("Left ragged" (make-line-with "paragraph mode" "left"))
	("Centered" (make-line-with "paragraph mode" "center"))
	("Right ragged" (make-line-with "paragraph mode" "right")))
    (-> "Margins"
	("Left margin" ... (interactive '("Left margin:") 'set-left-margin))
	("Right margin" ... (interactive '("Right margin:") 'set-right-margin))
	("First indentation" ...
	 (interactive '("First indentation:") 'set-first-indent)))
    (-> "Spacing"
	("Interline separation" ...
	 (interactive '("Separation between lines:") 'set-interline))
	("Interline space" ...
	 (interactive '("Space between lines:") 'set-interline-spc))
	("Interparagraph space" ...
	 (interactive '("Space between paragraphs:") 'set-interpar-spc)))
    (-> "Hyphenation"
	("Normal" (make-line-with "paragraph hyphenation" "normal"))
	("Professional"
	 (make-line-with "paragraph hyphenation" "professional")))
    (-> "Number of columns"
	("1" (make-line-with "nr columns" "1"))
	("2" (make-line-with "nr columns" "2"))
	("3" (make-line-with "nr columns" "3")))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The main Format menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define format-menu
  '((if (test-env? "mode" "text") (-> "Font" (link text-font-menu)))
    (if (test-env? "mode" "math") (-> "Font" (link math-font-menu)))
    (if (test-env? "mode" "prog") (-> "Font" (link prog-font-menu)))
    (-> "Color" (link color-menu))
    (-> "Language" (link language-menu))
    (if (test-env? "mode" "math")
	---
	(link math-format-menu))
    ---
    (link paragraph-menu)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Document menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define style-menu '())
(define use-package-menu '())

(define document-menu
  '((-> "Style"
	("None" (init-style "none"))
	---
	(link style-menu)
	---
	("Other" ... (interactive '("Document style:") 'init-style)))
    (-> "Use package"
	(link use-package-menu)
	---
	("Other" ... (interactive '("Use package:") 'init-extra-style)))
    (-> "Master"
	("Attach" (interactive '("Master file:") 'project-attach))
	("Detach" (project-detach)))
    ("Preamble mode" (toggle-preamble))
    ---
    (-> "Font"
	(-> "Text font"
	    ("Default" (init-default "font"))
	    ---
	    ("Roman" (init-env "font" "roman"))
	    ("Pandora" (init-env "font" "pandora"))
	    ("Concrete" (init-env "font" "concrete"))
	    ("Cyrillic" (init-env "font" "cyrillic"))
	    ---
	    ("Bookman" (init-env "font" "bookman"))
	    ("Courier" (init-env "font" "courier"))
	    ("Helvetica" (init-env "font" "helvetica"))
	    ("Palatino" (init-env "font" "palatino"))
	    ("Times" (init-env "font" "times"))
	    ---
	    ("Lucida" (init-env "font" "x-lucida"))
	    ("Utopia" (init-env "font" "x-utopia")))
	(-> "Math font"
	    ("Default" (init-default "math font"))
	    ---
	    ("Roman" (init-env "math font" "roman"))
	    ("Concrete" (init-env "math font" "concrete"))
	    ("Euler new roman" (init-env "math font" "ENR"))
	    ("Adobe" (init-env "math font" "adobe")))
	(-> "Prog font"
	    ("Default" (init-default "prog font"))
	    ---
	    ("Roman" (init-env "prog font" "roman"))
	    ("Pandora" (init-env "prog font" "pandora"))
	    ("Concrete" (init-env "prog font" "concrete"))
	    ("Times" (init-env "prog font" "times")))
	---
	(-> "Size"
	    ("Default" (init-default "font base size"))
	    ---
	    ("8" (init-env "font base size" "8"))
	    ("9" (init-env "font base size" "9"))
	    ("10" (init-env "font base size" "10"))
	    ("11" (init-env "font base size" "11"))
	    ("12" (init-env "font base size" "12"))
	    ("14" (init-env "font base size" "14"))
	    ---
	    ("Other" ... (interactive '("Font size:") 'init-font-size)))
	(-> "Dpi"
	    ("Default" (init-default "dpi"))
	    ---
	    ("150" (init-env "dpi" "150"))
	    ("200" (init-env "dpi" "200"))
	    ("300" (init-env "dpi" "300"))
	    ("400" (init-env "dpi" "400"))
	    ("600" (init-env "dpi" "600"))
	    ("800" (init-env "dpi" "800"))
	    ("1200" (init-env "dpi" "1200"))
	    ---
	    ("Other" ... (interactive '("Font dpi:") 'init-dpi))))
    (-> "Magnification"
	("Default" (init-default "magnification"))
	---
	("0.7" (init-env "magnification" "0.7"))
	("0.8" (init-env "magnification" "0.8"))
	("1" (init-env "magnification" "1"))
	("1.2" (init-env "magnification" "1.2"))
	("1.4" (init-env "magnification" "1.4"))
	("1.7" (init-env "magnification" "1.7"))
	("2" (init-env "magnification" "2"))
	---
	("Other" ... (interactive '("Magnification:") 'init-magn)))
    (-> "Color"
	(-> "Foreground"
	    ("Default" (init-default "color"))
	    ---
	    ("Black" (init-env "color" "black"))
	    ("White" (init-env "color" "white"))
	    ("Grey" (init-env "color" "grey"))
	    ("Red" (init-env "color" "red"))
	    ("Blue" (init-env "color" "blue"))
	    ("Yellow" (init-env "color" "yellow"))
	    ("Green" (init-env "color" "green"))
	    ("Orange" (init-env "color" "orange"))
	    ("Magenta" (init-env "color" "magenta"))
	    ("Brown" (init-env "color" "brown"))
	    ("Pink" (init-env "color" "pink")))
	(-> "Background"
	    ("Default" (init-default "background color"))
	    ---
	    ("Black" (init-env "background color" "black"))
	    ("White" (init-env "background color" "white"))
	    ("Grey" (init-env "background color" "grey"))
	    ("Red" (init-env "background color" "red"))
	    ("Blue" (init-env "background color" "blue"))
	    ("Yellow" (init-env "background color" "yellow"))
	    ("Green" (init-env "background color" "green"))
	    ("Orange" (init-env "background color" "orange"))
	    ("Magenta" (init-env "background color" "magenta"))
	    ("Brown" (init-env "background color" "brown"))
	    ("Pink" (init-env "background color" "pink"))
	    ("Light grey" (init-env "background color" "light grey"))
	    ("Dark grey" (init-env "background color" "dark grey"))
	    ("Broken white" (init-env "background color" "broken white"))))
    (-> "Language"
	("Default" (init-default "language"))
	---
	("British" (init-language "british"))
	("Czech" (init-language "czech"))
	("Dutch" (init-language "dutch"))
	("English" (init-language "english"))
	("French" (init-language "french"))
	("German" (init-language "german"))
	("Hungarian" (init-language "hungarian"))
	("Italian" (init-language "italian"))
	("Polish" (init-language "polish"))
	("Portuguese" (init-language "portuguese"))
	("Romanian" (init-language "romanian"))
	("Russian" (init-language "russian"))
	("Spanish" (init-language "spanish"))
	("Swedish" (init-language "swedish"))
	("Ukrainian" (init-language "ukrainian")))
    (-> "Paragraph"
	(-> "Style"
	    ("Default" (init-default "paragraph mode"))
	    ---
	    ("Justified" (init-env "paragraph mode" "justify"))
	    ("left ragged" (init-env "paragraph mode" "left"))
	    ("Centered" (init-env "paragraph mode" "center"))
	    ("Right ragged" (init-env "paragraph mode" "right")))
	(-> "Hyphenation"
	    ("Default" (init-default "paragraph hyphenation"))
	    ---
	    ("Normal" (init-env "paragraph hyphenation" "normal"))
	    ("Professional" (init-env "paragraph hyphenation" "professional")))
	(-> "Margins"
	    ("Default" (init-default "first indentation"))
	    ---
	    ("First indentation" ...
	      (interactive '("First indentation:") 'init-first-indent)))
	(-> "Spacing"
	    ("Default" (init-default "interline space" "line stretch"
				     "interpargraph space"))
	    ---
	    ("Interline separation" ...
	     (interactive '("Separation between lines:") 'init-interline))
	    ("Interline space" ...
	     (interactive '("Space between lines:") 'init-interline-spc))
	    ("Interparagraph space" ...
	     (interactive '("Space between paragraphs:") 'init-interpar-spc)))
	(-> "Number of columns"
	    ("Default" (init-default "nr columns"))
	    ---
	    ("1" (init-env "nr columns" "1"))
	    ("2" (init-env "nr columns" "2"))
	    ("3" (init-env "nr columns" "3"))))
    (-> "Page"
	(-> "Type"
	    ("Default" (check "*" (test-default? "page medium"))
	     (begin (init-default "page medium")
		    (set-page-parameters)))
	    ---
	    ("Paper" (set-page-medium "paper"))
	    ("Papyrus" (set-page-medium "papyrus"))
	    ("Automatic" (set-page-medium "automatic")))
	(-> "Size"
	    ("Default" (check "*" (test-default? "page type"))
	     (begin (init-default "page type")
		    (set-page-parameters)))
	    ---
	    ("A3" (set-page-type "a3"))
	    ("A4" (set-page-type "a4"))
	    ("A5" (set-page-type "a5"))
	    ("B4" (set-page-type "b4"))
	    ("B5" (set-page-type "b5"))
	    ("B6" (set-page-type "b6"))
	    ("Letter" (set-page-type "letter"))
	    ("Legal" (set-page-type "legal"))
	    ("Executive" (set-page-type "executive"))
	    ("Lecture note" (set-page-type "lecture note"))
	    ---
	    ("Other" ...
	      (interactive '("Page width:" "Page height:" "Text width:")
			   'init-page-size)))
	(-> "Orientation"
	    ("Default" (check "*" (test-default? "page orientation"))
	     (begin (init-default "page orientation")
		    (set-page-parameters)))
	    ---
	    ("Portrait" (set-page-orientation "portrait"))
	    ("Landscape" (set-page-orientation "landscape")))
	(-> "Layout"
	    ("Default" (init-default "odd page margin" "even page margin"
				     "page right margin" "page top margin"
				     "page bottom margin" "paragraph width"))
	    ---
	    ("Set margins" ...
	     (interactive
	       '("Left margin:" "Right margin:"
		 "Top margin:" "Bottom margin:") 'init-page-margins))
	    ("Set text width" ...
	      (interactive '("Text width:") 'init-text-width)))
	(-> "Breaking"
	    (-> "Algorithm"
		("Default" (init-default "page breaking"))
		---
		("Sloppy" (init-env "page breaking" "sloppy"))
		("Medium" (init-env "page breaking" "medium"))
		("Professional" (init-env "page breaking" "optimal")))
	    (-> "Limits"
		("Allowed reduction" ...
		 (interactive '("How much shorter may pages become?")
			      'init-page-shrink))
		("Allowed extension" ...
		 (interactive '("How much longer may pages become?")
			      'init-page-extend)))
	    (-> "Flexibility"
		("Default" (init-default "page flexibility"))
		---
		("0" (init-env "page flexibility" "0.0"))
		("1/4" (init-env "page flexibility" "0.25"))
		("1/2" (init-env "page flexibility" "0.5"))
		("3/4" (init-env "page flexibility" "0.75"))
		("1" (init-env "page flexibility" "1.0"))
		---
		("Other" ...
		 (interactive '("Flexibility factor for vertical spacing: ")
			      'init-page-flexibility))))
	(-> "Screen layout"
	    ("Default" (init-default "reduction page left margin"
				     "reduction page right margin"
				     "reduction page top margin"
				     "reduction page bottom margin"))
	    ---
	    ("Margins as on paper" (init-as-on-paper))
	    ("Reduce margins" ... (interactive
	      '("Reduce left margin by:"
		"Reduce right margin by:"
		"Reduce top margin by:"
		"Reduce bottom margin by:") 'init-screen-reduction))
	    ("Show header and footer" (toggle-visible-header-and-footer))))
    ---
    (-> "Update"
	("All" (begin (generate-all-aux) (update-buffer)))
	---
	("Buffer" (update-buffer))
	("Bibliography" (begin (generate-all-aux) (update-buffer)))
;;	("Bibliography" (generate-aux "bibliography"))
	("Table of contents" (generate-aux "table-of-contents"))
	("Index" (generate-aux "index"))
	("Glossary" (generate-aux "glossary")))))
